{--------------------------------------------------------}
{                                                        }
{    Programa de acesso a redes sociais                  }
{                                                        }
{    Mdulo de folheamento de status e direct messages   }
{                                                        }
{    Autor: Jos Antonio Borges                          }
{                                                        }
{    Em setembro/2010                                    }
{                                                        }
{--------------------------------------------------------}

unit twexibe;

interface

uses
    windows, dvcrt, dvwin, dvForm, twjson, twmsg, twpost, twimg, twvars,
    sysutils, shellapi, dateUtils, SuperObject;

procedure folhearInfoUser (pusr: PUser);
procedure folhearStatuses (resp: string; opcao: char; var continua: boolean);
procedure folhearInfoStatus (pst: PStatus);
procedure folhearDirectMessages (resp: string; var continua: boolean);
procedure folhearDirectMessage (pdm: PDirectMessage);

implementation

function portug (s: string): string;
begin
    result := s;
    if ansiUpperCase(s) = 'TRUE'  then result := 'Sim'
    else
    if ansiUpperCase(s) = 'FALSE' then result := 'No';
end;

function GMTToLocalTime(GMTTime: TDateTime): TDateTime;
var
  GMTST: TSystemTime;
  LocalST: TSystemTime;
begin
   SysUtils.DateTimeToSystemTime(GMTTime, GMTST);
   SysUtils.Win32Check(Windows.SystemTimeToTzSpecificLocalTime(nil, GMTST, LocalST));
   Result := SysUtils.SystemTimeToDateTime(LocalST);
end;

function convData (created_at: string): string;
const
    meses: string = 'JanFebMarAprMayJunJulAugSepOctNovDec';
    semanaPortugues: string = 'SegundaTerca  Quarta Quinta Sexta  Sbado Domingo';

var dt, dt2: TDateTime;
    dia, mes, ano, hora, min, seg, ms: word;
    semana: word;
    sem: string;

    function int2ToStr(n: integer): string;
    begin
        if n <= 9 then result := '0' + intToStr(n)
                 else result := intToStr(n);
    end;

begin
    created_at := trim(created_at);
    try
        dia := strToInt (copy (created_at, 9, 2));
        mes := (pos(copy(created_at, 5, 3), meses) div 3) + 1;
        ano := strToInt (copy (created_at, length(created_at)-3, 4));
        hora := strToInt (copy (created_at, 12, 2));
        min := strToInt (copy (created_at, 15, 2));
        seg := strToInt (copy (created_at, 18, 2));
        ms := 0;
        dt := encodeDate (ano, mes, dia) +
              encodeTime(hora, min, seg, ms)
    except
        dt := Now;
    end;

    dt2 := GMTToLocalTime(dt);

    decodeDate (dt2, ano, mes, dia);
    decodeTime (dt2, hora, min, seg, ms);

    semana := dayOfTheWeek (dt2)-1;
    sem := copy (semanaPortugues, semana*7+1, 7);

    result := int2ToStr(hora) + ':' + int2ToStr(min) + ' ' + sem + ' ' +
              int2ToStr(dia)+'/'+int2ToStr(mes)+'/'+intToStr(ano);
end;

{--------------------------------------------------------}

procedure folhearInfoUser (pusr: PUser);
var
    vcampos: array [1..15] of shortString;
    ncampos: integer;

    procedure criaCampo (nomeSom, valor: string);
    begin
        ncampos := ncampos + 1;
        vcampos[ncampos] := valor;
        formCampo(nomeSom, pegaTextoMensagem(nomeSom), vcampos[ncampos], 255);
    end;

begin
    gotoxy (1, 6);
    limpaBaixo;

    TextBackground (BLUE);
    mensagem ('TWEDESCT', 1);  {'Confira com as setas, ESC termina'}
    TextBackground (BLACK);
    ncampos := 0;
    formCria;
    with pusr^ do
        begin
            criaCampo('TW_NOME' , name);                 {'Nome: '}
            criaCampo('TW_APELI', screen_name);          {'Apelido: '}
            criaCampo('TW_LOCAL', location);             {'Localizao: '}
            criaCampo('TW_DESCR', description);          {'Descrio: '}
            criaCampo('TW_IDENT', id);                   {'Identificao: '}
            criaCampo('TW_LOCIM', profile_image_url);    {'Local da imagem: '}
            criaCampo('TW_HPAGE', url);                  {'Homepage: '}
            criaCampo('TW_SEGUI', followers_count);      {'Seguidores:'}
            criaCampo('TW_AMIGS', friends_count);        {'Amigos: '}
            criaCampo('TW_CRIAD', convData(created_at)); {'Criado em '}
            criaCampo('TW_FAVRS', favourites_count);     {'Favoritos: '}
            criaCampo('TW_SEGND', following);            {'Seguindo: '}
            criaCampo('TW_NTWEE', statuses_count);       {'Quantos tweets: '}
            criaCampo('TW_LINGU', lang);                 {'Linguagem: '}
        end;

    mostraImagem(pusr^.profile_image_url, windowSize.x-140, windowSize.y-140);
    formEdita(false);
end;

{--------------------------------------------------------}

procedure folhearInfoStatus (pst: PStatus);
var
    s: string;
    vcampos: array [1..15] of shortString;
    ncampos: integer;

    procedure criaCampo (nomeSom, valor: string);
    begin
        ncampos := ncampos + 1;
        vcampos[ncampos] := valor;
        formCampo(nomeSom, pegaTextoMensagem(nomeSom), vcampos[ncampos], 255);
    end;

begin
    limpaBaixo;
    TextBackground (BLUE);
    mensagem ('TWEDESCT', 1);  {'Confira com as setas, ESC termina'}
    TextBackground (BLACK);
    ncampos := 0;
    formCria;
    with pst^ do
        begin
            criaCampo('TW_TEXT',  text);                    {'Texto: '}
            criaCampo('TW_AUTOR', user^.name);              {'Autor: '}
            s := source;
            if pos('>', s) <> 0 then delete (s, 1, pos('>', s));
            if pos('<', s) <> 0 then delete (s, pos('<', s), 999);
            criaCampo('TW_SRC',   s);                       {'Origem: '}
            criaCampo('TW_CREAT', convData(created_at));    {'Criado em: '}
            criaCampo('TW_ID',    id);                      {'Identificao: '}
            criaCampo('TW_RETWI', portug(retweeted));       {'Retuitado: '}
            criaCampo('TW_RTCNT', retweet_count);           {'Contador de retwits: }
            criaCampo('TW_TRUNC', portug(truncated));       {'Truncado: '}
            criaCampo('TW_FAVOR', portug(favorited));       {'Favorito: '}
            criaCampo('TW_INRES', in_reply_to_status_id);   {'Resposta ao status: '}
            criaCampo('TW_INREU', in_reply_to_user_id);     {'Resposta ao usurio: '}
            criaCampo('TW_REPSN', in_reply_to_screen_name); {'Resposta ao apelido: '}
        end;
    formEdita(false);
end;

{--------------------------------------------------------}

function selSetasFolhearStatus: char;
var n: integer;
const
    opmenu: string = 'TEUMSXLR' + #$1b;
begin
    popupMenuCria(wherex, wherey, 50, 9, RED);
    MenuAdiciona ('TWEX_T');   {'T - Retuitar'}
    MenuAdiciona ('TWEX_E');   {'E - Replicar'}
    MenuAdiciona ('TWEX_U');   {'U - Folhear usurio'}
    MenuAdiciona ('TWEX_M');   {'M - Folhear as informaes da mensagem'}
    MenuAdiciona ('TWEX_S');   {'S - Salvar mensagens em arquivo'}
    MenuAdiciona ('TWEX_X');   {'X - Executar a referncia http'}
    MenuAdiciona ('TWEX_L');   {'L - Ler editorando a mensagem'}
    MenuAdiciona ('TWEX_R');   {'R - Remover a mensagem'}
    MenuAdiciona ('TWOP_ESC'); {'ESC - terminar'}

    n := popupMenuSeleciona;
    if (n < 1) then
        result := #$0
    else
        result := opmenu[n];
end;

{--------------------------------------------------------}

procedure salvaInfoStatus (psts: LStatuses; n: integer);
var c, c2: char;
    n1, n2, i: integer;
    nome, s: string;
    arq: TextFile;
    pst: PStatus;

label desist;
begin
    mensagem ('TWNOMSLV', 1);    {'Editore o nome do arquivo a salvar: '}
    c := sintEdita (nomeArqSalva, wherex, wherey, 200, true);
    writeln;
    if (c = ESC) or (nomeArqSalva = '') then goto desist;

    mensagem ('TWTSTOD', 0);     {'Aperte T para todos, outra tecla s para esse: '}
    sintLeTecla (c, c2);
    writeln;
    if c = ESC then goto desist;

    if upcase(c) = 'T' then
        begin
            n1 := 0;
            n2 := psts.Count-1;
        end
    else
        begin
            n1 := n;
            n2 := n;
        end;

    assignFile (arq, nomeArqSalva);
    {$I-}  append (arq);  {$I+}
    if ioresult <> 0 then
        {$I-}  rewrite (arq);  {$I+}
    if ioresult <> 0 then
        begin
            mensagem ('TWERRGRV', 1);  {'Erro ao gravar arquivo'}
            exit;
        end;

    for i := n1 to n2 do
        begin
            pst := PStatus(psts[i]);
            with pst^ do
                begin
                    nome := copy (user^.screen_name + ',                 ', 1, 20);
                    s := nome + ' s ' + convData (created_at);
                    writeln (arq, s);
                    writeln (arq, text);
                end;
        end;

    closeFile (arq);
    exit;

desist:
    mensagem ('TWDESIST', 1);   {'Desistiu...'}
end;

{--------------------------------------------------------}

procedure executaHttp (s: string);
var s2: string;
    i, p: integer;
    c: char;
begin
    s2 := ansiUpperCase(s);
    p := pos ('HTTP://', s2);
    if p = 0 then
        p := pos ('HTTPS://', s2);
    if p = 0 then
        begin
            mensagem ('TWSITNAO', 1); {'Site no foi localizado na mensagem'}
            exit;
        end;

    delete (s, 1, p-1);
    for i := 1 to length(s) do
        if s[i] = ' ' then
            begin
                delete (s, i, 999);
                break;
            end;

    mensagem ('TWF12', 1);     {'Aperte F11 para executar pelo Windows, F12 para executar pelo DOSVOX'}
    mensagem ('TWENTRET', 1);  {'Aperte enter depois que retornar'}

    c := sintEdita (s, wherex, wherey, 200, false);
    if c = F11 then
        shellExecute (0, 'open', pchar(s), NIL, NIL, SW_SHOWNORMAL);
end;

{--------------------------------------------------------}

procedure editora (s: string);
begin
    writeln;
    mensagem ('TWEDITOR', 2);  {'Soletre ou editore'}
    sintEdita (s, wherex, wherey, 200, false);
end;

{--------------------------------------------------------}

procedure folhearStatuses (resp: string; opcao: char; var continua: boolean);
var
    psts: LStatuses;
    pst: PStatus;
    i, j: integer;
    s: string;
    nome: string;
    c, c2: char;
    nitem: integer;
    ok: boolean;
    obj: ISuperObject;

label libMemoria, executa;
begin
    continua := false;
    gotoxy (1, 5);
    limpaBaixo;

    obj := criaSuperObject (resp);
    if opcao = 'S' then
        psts := montaStatuses (obj)
    else
        psts := montaStatuses (obj['statuses']);

    if psts.Count = 0 then
         begin
             mensagem ('TWNAOENC', 1);  {'No encontrado.'}
             exit;
         end;

    TextBackground (BLUE);
    mensagem ('TWFSETAS', 1);  {'Folheie com as setas, F9 d opes, ESC termina'}
    TextBackground (BLACK);

    folheiaCria (1, 7, 80, 24);
    for i := 0 to psts.Count-1 do
        begin
            pst := PStatus(psts[i]);
            with pst^ do
                begin
                    nome := copy (user^.screen_name + ',                 ', 1, 20);
                    s := nome + ' s ' + convData (created_at);
                    folheiaAdiciona(s);
                    s := text;
                    for j := 1 to length (s) do
                       if s[j] < ' ' then s[j] := '';
                    folheiaAdiciona('   '+s);
                end;
        end;

    if psts.Count >= 20 then
        folheiaAdiciona('-- mais twits --');

    nitem := 1;

    repeat
        limpaBufTec;
        ok := folheiaExecuta(nitem, nitem, c, c2, true);
        if not ok then break;

        if (nitem-1) div 2 >= psts.Count then
            begin
                continua := true;
                break;
            end;

        pst := PStatus(psts[(nitem-1) div 2]);
        gotoxy (1, 5);
        limpaBaixo;
        textColor (YELLOW);
        write (pst^.text);
        textColor (WHITE);

        gotoxy (1, 7);
        if c = #0 then
           begin
               case c2 of
                   F1:   begin
                             gotoxy (1, 25);
                             mensagem ('TWFSETAS', 0);  {'Folheie com as setas, F9 d opes, ESC termina'}
                         end;
                   F9:   begin
                             c := selSetasFolhearStatus;
                             goto executa;
                         end;
               end
           end
        else
           begin
executa:
               case upcase(c) of
                   'T': retuitar (pst);
                   'E': replicar (pst);
                   'U': folhearInfoUser (pst^.user);
                   'M': folhearInfoStatus (pst);
                   'S': salvaInfoStatus (psts, (nitem-1) div 2);
                   'X': executaHttp (pst^.text);
                   'R': destruir(pst^.id);
                   ENTER,
                   'L': editora (pst^.text);
               else
                   sintBip;
               end;
           end;

        gotoxy (1, 5);
        limpaBaixo;
        TextBackground (BLUE);
        write (pegaTextoMensagem ('TWFSETAS'));  {'Folheie com as setas, F9 d opes, ESC termina'}
        TextBackground (BLACK);

    until c = ESC;

libMemoria:
    folheiaDestroi;

    for i := 0 to psts.Count-1 do
        begin
            pst := PStatus(psts[i]);
            dispose (pst.user);
            pst.user := NIL;
            dispose (pst);
        end;

    psts.Clear;
end;

{--------------------------------------------------------}

procedure folhearDirectMessage (pdm: PDirectMessage);
var
    vcampos: array [1..15] of shortString;
    ncampos: integer;

    procedure criaCampo (nomeSom, valor: string);
    begin
        ncampos := ncampos + 1;
        vcampos[ncampos] := valor;
        formCampo(nomeSom, pegaTextoMensagem(nomeSom), vcampos[ncampos], 255);
    end;

begin
    limpaBaixo;
    TextBackground (BLUE);
    mensagem ('TWEDESCT', 1);  {'Confira com as setas, ESC termina'}
    TextBackground (BLACK);
    ncampos := 0;
    formCria;
    with pdm^ do
        begin
            criaCampo('TW_TEXT',  text);                    {'Texto: '}
            criaCampo('TW_ENVIA', sender_screen_name);      {'Enviado por: '}
            criaCampo('TW_RECEB', recipient_screen_name);   {'Recebido por: '}
            criaCampo('TW_CREAT', convData(created_at));    {'Criado em: '}
            criaCampo('TW_ID',    id);                      {'Identificao: '}
        end;

    formEdita(false);
end;

{--------------------------------------------------------}

function selSetasFolhearDirectMessages: char;
var n: integer;
const
    opmenu: string = 'EUDMSXLR' + #$1b;
begin
    popupMenuCria(wherex, wherey, 50, 9, RED);
    MenuAdiciona ('TWEX_E');    {'E - Replicar a mensagem'}
    MenuAdiciona ('TWEX_U');    {'U - Folhear o usurio'}
    MenuAdiciona ('TWEX_D');    {'D - Folhear o destinatrio'}
    MenuAdiciona ('TWEX_M');    {'M - Folhear a mensagem'}
    MenuAdiciona ('TWEX_S');    {'S - Salvar mensagens em arquivo'}
    MenuAdiciona ('TWEX_X');    {'X - Executar a referncia http'}
    MenuAdiciona ('TWEX_L');    {'L - Ler editorando a mensagem'}
    MenuAdiciona ('TWEX_R');    {'R - Remover a mensagem'}
    MenuAdiciona ('TWOP_ESC');  {'ESC - terminar'}

    n := popupMenuSeleciona;
    if (n < 1) then
        result := #$0
    else
        result := opmenu[n];
end;

{--------------------------------------------------------}

procedure salvaDirectMessages (pdms: LDirectMessages; n: integer);
var
    c, c2: char;
    n1, n2, i: integer;
    nomes, s: string;
    arq: TextFile;
    pdm: PDirectMessage;

label desist;
begin
    mensagem ('TWNOMSLV', 1);    {'Editore o nome do arquivo a salvar: '}
    c := sintEdita (nomeArqSalva, wherex, wherey, 200, true);
    writeln;
    if (c = ESC) or (nomeArqSalva = '') then goto desist;

    mensagem ('TWTSTOD', 0);     {'Aperte T para todos, outra tecla s para esse: '}
    sintLeTecla (c, c2);
    writeln;
    if c = ESC then goto desist;

    if upcase(c) = 'T' then
        begin
            n1 := 0;
            n2 := pdms.Count-1;
        end
    else
        begin
            n1 := n;
            n2 := n;
        end;

    assignFile (arq, nomeArqSalva);
    {$I-}  append (arq);  {$I+}
    if ioresult <> 0 then
        {$I-}  rewrite (arq);  {$I+}
    if ioresult <> 0 then
        begin
            mensagem ('TWERRGRV', 1);  {'Erro ao gravar arquivo'}
            exit;
        end;

    for i := n1 to n2 do
        begin
            pdm := PDirectMessage(pdms[i]);
            with pdm^ do
                begin
                    nomes := copy (sender_screen_name + ',                 ', 1, 20) + ' para ' +
                            copy (recipient_screen_name + ',                 ', 1, 20);
                    s := nomes + ' s ' + convData (created_at);
                    folheiaAdiciona(s);
                    folheiaAdiciona('   '+text);
                end;
        end;

    closeFile (arq);
    exit;

desist:
    mensagem ('TWDESIST', 1);   {'Desistiu...'}
end;

{--------------------------------------------------------}

procedure folhearDirectMessages (resp: string; var continua: boolean);
var
    pdms: LDirectMessages;
    pdm: PDirectMessage;
    i: integer;
    s: string;
    nomes: string;
    c, c2: char;
    nitem: integer;
    ok: boolean;
    obj: ISuperObject;

label libMemoria, executa;
begin
    continua := false;
    gotoxy (1, 5);
    limpaBaixo;

    obj := criaSuperObject (resp);
    pdms := montaDirectMessages (obj);
    if pdms.Count = 0 then
         begin
             mensagem ('TWNAOENC', 1);  {'No encontrado.'}
             exit;
         end;

    TextBackground (BLUE);
    mensagem ('TWFSETAS', 1);  {'Folheie com as setas, F9 d opes, ESC termina'}
    TextBackground (BLACK);

    folheiaCria (1, 7, 80, 24);

    for i := 0 to pdms.Count-1 do
        begin
            pdm := PDirectMessage(pdms[i]);
            with pdm^ do
                begin
                    nomes := copy (sender_screen_name + ',                 ', 1, 20) + ' para ' +
                            copy (recipient_screen_name + ',                 ', 1, 20);
                    s := nomes ;////////////////+ ' s ' + convData (created_at);
                    folheiaAdiciona(s);
                    folheiaAdiciona('   '+text);
                end;
        end;
    if pdms.Count >= 20 then
        folheiaAdiciona('-- mais twits --');

    nitem := 1;

    repeat
        limpaBufTec;
        ok := folheiaExecuta(nitem, nitem, c, c2, true);
        if not ok then break;

        if (nitem-1) div 2 >= pdms.Count then
            begin
                continua := true;
                break;
            end;

        pdm := PDirectMessage(pdms[(nitem-1) div 2]);
        gotoxy (1, 5);
        limpaBaixo;
        textColor (YELLOW);
        write (pdm^.text);
        textColor (WHITE);

        gotoxy (1, 7);
        if c = #0 then
            begin
                case c2 of
                    F1:   begin
                              gotoxy (1, 25);
                              mensagem ('TWFSETAS', 0);  {'Folheie com as setas, F9 d opes, ESC termina'}
                          end;
                    F9:   begin
                              c := selSetasFolhearDirectMessages;
                              goto executa;
                          end;
                end
            end
        else
            begin
executa:
                case upcase(c) of
                    'E': replicarMsgDireta (pdm);
/////////////                    'U': folhearInfoUser (pdm^.sender);
/////////////                    'D': folhearInfoUser (pdm^.recipient);
                    'M': folhearDirectMessage (pdm);
                    'S': salvaDirectMessages (pdms, (nitem-1) div 2);
                    'X': executaHttp (pdm^.text);
                    'L': editora (pdm^.text);
                    'R': destruir(pdm^.id);
                end;
            end;

        gotoxy (1, 5);
        limpaBaixo;
        TextBackground (BLUE);
        write (pegaTextoMensagem ('TWFSETAS'));  {'Folheie com as setas, F9 d opes, ESC termina'}
        TextBackground (BLACK);

    until c = ESC;

libMemoria:
    folheiaDestroi;

    for i := 0 to pdms.Count-1 do
        begin
            pdm := PDirectMessage(pdms[i]);
//////////////////            dispose (pdm.sender);
//////////////////            pdm.sender := NIL;
//////////////////            dispose (pdm.recipient);
//////////////////            pdm.recipient := NIL;
            dispose (pdm);
        end;

    pdms.Clear;
end;

end.

